# vim:set ts=2 sw=2 sts=2 et:
from rzn.api import Logger
import os
import os.path
import subprocess
import sys
import errno
import re

log = Logger(__name__)


def parse_hglog(log):
  log = unicode(log, "utf-8")
  log = log.split("\n\n")
  nlog = []
  for l in log:
    ld = l.split("\n")
    if len(ld) == 1:
      continue
    loglines = [re.match("(.*): +(.*)", ll) for ll in ld]
    ld = dict([match.groups() for match in loglines if match])
    #
    if "changeset" in ld:
      rev, cs = ld["changeset"].split(":")
      ld["changeset"] = cs
      ld["rev"] = int(rev)
      nlog.append(ld)
  return nlog

HG_EXE = None


def _gethg():
  global HG_EXE
  paths = ["/usr/bin", "/usr/local/bin"]
  for p in paths:
    fn = os.path.join(p, "hg")
    if os.path.exists(fn):
      log.info("Found HG at %s" % fn)
      HG_EXE = fn
      return
  log.warning("HG not found in default search paths")
  HG_EXE = "hg"


def hg(*opts):
  kwargs = {}
  if sys.platform == "win32":
    import _subprocess
    import locale
    si = subprocess.STARTUPINFO()
    si.dwFlags |= _subprocess.STARTF_USESHOWWINDOW
    si.wShowWindow = _subprocess.SW_HIDE
    kwargs["startupinfo"] = si
    opts = [o.encode(locale.getpreferredencoding()) for o in opts]

  if HG_EXE is None:
    _gethg()

  # Fix for locale problems
  localefix = os.environ.copy()
  localefix["LC_ALL"] = "en_US.UTF-8"
  localefix["LANGUAGE"] = "en_US.UTF-8"

  cmd = [HG_EXE]+list(opts)
  while 1:

    # Fix for read in subprocess (on Mac only?). EINTR not handled by Python IO
    # Correctly, so retry call when we get the signal. Might be fixed in newer
    # releases?
    if sys.platform == "darwin":
      # env borked on py2app'd executable?
      hg = subprocess.Popen(cmd, stderr=subprocess.STDOUT,
                                 stdout=subprocess.PIPE, env={}, **kwargs)
    else:
      hg = subprocess.Popen(cmd, stderr=subprocess.STDOUT,
                                 stdout=subprocess.PIPE, env=localefix, **kwargs)
    try:
      out, err = hg.communicate()
      return out, hg.returncode
    except IOError as e:
      if e.errno == errno.EINTR:
        log.warning("hg call failed with EINTR, retrying")
      else:
        raise
